import os
import platform
import shutil
from pathlib import Path

DOIT_CONFIG = {
    "default_tasks": ["format", "lint", "test", "prepare_windows_build"],
    "backend": "json",
}

HERE = Path(__file__).parent


def task_format():
    """Reformat all files using black."""
    print(HERE)
    return {"actions": ["isort -y ", ["black", HERE]], "verbosity": 1}


def task_format_check():
    """Check, but not change, formatting using black."""
    return {"actions": [["black", HERE, "--check"]], "verbosity": 1}


def task_lint():
    """Lint all files with Prospector."""
    return {"actions": [["prospector"]], "verbosity": 1}


def task_test():
    """Run Pytest with coverage."""
    return {
        "actions": [["pytest", "--cov=mad_gui", "--cov-config=.coveragerc", "-vv"]],
        "verbosity": 2,
    }


def task_prepare_windows_build():
    """Build a standalone windows executable."""
    commands = []

    import sys
    venv_path=sys.executable.split(os.sep)

    import warnings
    try:
        answer = input("For more information about this message see"\
                       "https://github.com/mad-lab-fau/mad-gui/blob/main/docs/developer_guidelines.rst#6-creating-an-executable."\
                       f"\n Go on with {os.sep.join(venv_path)} as the virtual environment exclusively used for packaging? (y/n):")
    except EOFError:
        answer = 'y'

    if answer.lower() == 'n':
        return



    def get_dst_path(path_venv: str):
        print(Path(os.sep.join(venv_path[:-2])) / "Lib/site-packages/mad_gui/qt_designer/build/")
        return Path(os.sep.join(venv_path[:-2])) / "Lib/site-packages/mad_gui/qt_designer/build/"

    def set_up_paths(path_venv):
        print(f"checking {get_dst_path(path_venv)}")
        if not os.path.exists(get_dst_path(path_venv)):
            raise FileNotFoundError("Apparently mad_gui is not installed in this environemnt. Use `pip install . ` to do so.")
        dst_path = get_dst_path(path_venv)
        os.makedirs(dst_path, exist_ok=True)

    def convert_ui_to_py(path_venv):
        dst_path = get_dst_path(path_venv)
        ui_files = [file for file in os.listdir(dst_path.parent) if ".ui" in file]
        for file in ui_files:
            os.popen(f"pyside2-uic -o {dst_path}\\{file.split('.')[0]}.py {dst_path.parent}\\{file}")

    return {
        "actions": [set_up_paths, convert_ui_to_py],
        "params": [{"name": "path_venv", "short": "v", "default": ".venv"}],
        "verbosity": 2,
    }


def task_docs():
    """Build the documentation."""
    # Copy the README into the docs folder
    shutil.copy(str(HERE / "README.md"), str(HERE / "docs"))
    # Delete Autogenerated files from previous run
    shutil.rmtree(str(HERE / "docs/_build"), ignore_errors=True)
    # Copy the images into the docs folder
    os.makedirs(HERE / "docs/_build/html/images/")
    for file in list((HERE / "docs/res/images/").glob("*")):
        shutil.copy(str(file), str(HERE / "docs/_build/html/images/" / str(file).split(os.sep)[-1]))

    # Copy the image buttons
    for file in list((HERE / "mad_gui/qt_designer/images").glob("*.png")):
        shutil.copy(str(file), str(HERE / "docs/_build/html/images/" / str(file).split(os.sep)[-1]))

    if platform.system() == "Windows":
        return {"actions": [[HERE / "docs/make.bat", "html"]], "verbosity": 2}
    else:
        return {"actions": [["make", "-C", HERE / "docs", "html"]], "verbosity": 2}
